package i2c_client;

# I2C-Client , 
# Kapselt die I2C-Funktionsaufrufe. 
# (c) DH1AAD / Ingo Gerlach  
# 
# 
# To DO 
#
# 0.1   07.09.2001 	Start 


use strict;
use vars qw($VERSION @ISA @EXPORT_OK $AUTOLOAD);
use i2c_ser;
use Socket;
use IO::Socket;

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
@EXPORT_OK = qw() ;
$VERSION = '0.1';

#------------------------------------------------------------------------------------#
#---------------------------    Funktions Konstanten  -------------------------------#
#------------------------------------------------------------------------------------#

 my $C_QUIT		= "0000";
 my $C_CHECK		= "0001";
 my $C_INIT		= "1000";
 my $C_REINIT		= "1001";
 my $C_PCF8591_INIT	= "2000";
 my $C_PCF8591_READ	= "2001";
 my $C_PCF8591_READ4	= "2002";
 my $C_PCF8591_AOUT	= "2003";
 my $C_PCF8591_SETDA	= "2004";
 my $C_PCF8591_DELETE	= "2005";
 my $C_PCF8574_TX	= "3000";
 my $C_PCF8574_RX	= "3001";
 my $C_LCD_INIT		= "3100";
 my $C_LCD_BACKLIGHT	= "3101"; 
 my $C_LCD_INSTR	= "3102"; 
 my $C_LCD_CADR		= "3103"; 
 my $C_LCD_WCHAR	= "3104";   
 my $C_LCD_WSTR		= "3105";   
 my $C_LCD_RCHAR	= "3106";   
 my $C_LCD_RSTR		= "3107";   
 my $C_SDA3302_INIT	= "4000";   
 my $C_SDA3302_CALC	= "4001";   
 my $C_SDA3302_SEND	= "4002";   
 my $C_SDA3302_DELETE	= "4003";    
 my $C_TSA5523_INIT	= "5000";   
 my $C_TSA5523_CALC	= "5001";   
 my $C_TSA5523_SEND	= "5002";   
 my $C_TSA5523_READ	= "5003";   
 my $C_TSA5523_DELETE	= "5004";    
 my $C_TSA5511_INIT	= "6000";   
 my $C_TSA5511_CALC	= "6001";   
 my $C_TSA5511_SEND	= "6002";   
 my $C_TSA5511_READ	= "6003";   
 my $C_TSA5511_DELETE	= "6004";    
 my $C_TSA6057_INIT	= "7000";   
 my $C_TSA6057_CALC	= "7001";   
 my $C_TSA6057_SEND	= "7002";   
 my $C_TSA6057_DELETE	= "7003";    

 my $socket_name 	= '/tmp/i2c_device';
 my $command;
 my $SERVER		= 1;	# Server benutzen 1, 0 direkt 
 my $PORT		= 0;	# falls != 0 tcp, sonst Socket
 
 # Hardware 
 my $I2C_INIT    	= 0;		# True , wenn Init OK  
 my $I2C_PORTDELAY	= 10;		# Hardware  
 my $I2C_TYPE		= "SER";	# Hardware  
 my $I2C_port 		= 0;		# Port, automatisch suchen

 my %PCF		= ("0"=>-1,"2"=>-1,"4"=>-1,"6"=>-1,"8"=>-1,"10"=>-1,"12"=>-1,"14"=>-1);
 my %SDA3302		= ("0"=>-1,"2"=>-1,"4"=>-1);
 my %TSA5523		= ("0"=>-1,"2"=>-1,"4"=>-1,"6"=>-1);
 my %TSA5511		= ("0"=>-1,"2"=>-1,"4"=>-1,"6"=>-1);
 my %TSA6057		= ("0"=>-1,"2"=>-1);
 
#------------------------------------------------------------------------------------#
#------------------------    Allgemeine I2C- Funktionen  ----------------------------#
#------------------------------------------------------------------------------------#

# Initialisiert den Client, Parameter
# Rckgabe Ser-Port bzw. -1 falls Server nicht aktiv, 
# 0 wenn kein I2C-Interface gefunden wurde.
sub init($) {
my $servuse 	= $_[0];
#my $port	= $_[1];
my $ret;
 if ( $servuse == 1) {
  $command = $C_INIT.":";
  $ret = send2server($command);
  $SERVER = 1;
 } else {
    $SERVER = 0;
    i2c_ser::set_port_delay($I2C_PORTDELAY); 
    # Interface automatisch suchen 
    $I2C_port = i2c_ser::init_iic($I2C_port); 
    $ret = ( $I2C_port > 0 ? $I2C_port : 0);   
   } 
  $I2C_INIT = $ret; 
 return $ret;
}

# Fhrt ein reinit i2c auf dem Server aus.
sub re_init() {
my $ret;
 if($SERVER) {
  $command = $C_REINIT.":";
  $ret = send2server($command);
 } else { $ret= init($SERVER); }
 return $ret;
}

#------------------------------------------------------------------------------------#
#-----------------------------  Funktionen PLL TSA6057  -----------------------------#
#------------------------------------------------------------------------------------#

# Gibt die def. Konstanten im Array zurck
sub tsa6057_const() {
 return i2c_ser::tsa6057_const();
}

# Initialisiert TSA6057
# Parameter Mode , Adresse
sub tsa6057_init($ $ $) {
my $raster	= $_[0];
my $mode	= $_[1];
my $adresse	= $_[2];
my $ret;
  if($SERVER) {
   $command = $C_TSA6057_INIT.":".$raster.":".$mode.":".$adresse.":";
   $ret = send2server($command);
  } else { 
    if (($I2C_INIT) && ($TSA6057{$adresse} == -1)) { # Pointer nur einmal anlegen
     $TSA6057{$adresse} =i2c_ser::create6057(); 
     i2c_ser::tsa6057_init($TSA6057{$adresse},$raster,$mode); 
     $ret = 1;
    } else { $ret = -1; }  
  } 
 return $ret;
}

# Berechnet die Teiler TSA6057
# Parameter Freq, Raster, Adresse
sub tsa6057_calc($ $ $) {
my $freq	= $_[0];
my $raster	= $_[1];
my $adresse	= $_[2];
my $ret;
 if($SERVER) {
  $command = $C_TSA6057_CALC.":".$freq.":".$raster.":".$adresse.":";
  $ret = send2server($command);
 } else { 
    if (($I2C_INIT) && ($TSA6057{$adresse} != -1)) { 
     $ret = i2c_ser::tsa6057_calc($TSA6057{$adresse},$freq,$raster); 
    } else { $ret = -1; }  
   }
 return $ret;
}

# Sendet die Teiler TSA6057
# Parameter Adresse
sub tsa6057_send($) {
my $adresse	= $_[0];
my $ret;
  if($SERVER) {
   $command = $C_TSA6057_SEND.":".$adresse.":";
   $ret = send2server($command);
  } else {
    if (($I2C_INIT) && ($TSA6057{$adresse} != -1)) { 
     $ret = i2c_ser::tsa6057_send($TSA6057{$adresse},$adresse); 
    } else { $ret = -1; }  
   }  
 return $ret;
}

# Lscht den Pointer auf TSA6057
# Parameter Adresse
sub tsa6057_delete($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_TSA6057_DELETE.":".$adresse.":";
  $ret = send2server($command);
 } else {
   if (($I2C_INIT) && ($TSA6057{$adresse} != -1)) {
    i2c_ser::delete6057($TSA6057{$adresse});
    $TSA6057{$adresse} = -1;
    $ret = 1;
   } else { $ret = -1; }
  }
 return $ret;
}

#------------------------------------------------------------------------------------#
#-----------------------------  Funktionen PLL TSA5511  -----------------------------#
#------------------------------------------------------------------------------------#

# Gibt die def. Konstanten im Array zurck
sub tsa5511_const() {
 return i2c_ser::tsa5511_const();
}

# Initialisiert TSA5511
# Parameter Mode , Adresse
sub tsa5511_init($ $ $) {
my $mode	= $_[0];
my $refquarz	= $_[1];
my $adresse	= $_[2];
my $ret;
 if($SERVER) {
  $command = $C_TSA5511_INIT.":".$mode.":".$refquarz.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($TSA5511{$adresse} == -1)) { # Pointer nur einmal anlegen
     $TSA5511{$adresse} =i2c_ser::create5511(); 
     i2c_ser::tsa5511_init($TSA5511{$adresse},$mode,$refquarz); 
     $ret = 1;
    } else { $ret = -1; }
  } 
 return $ret;
}

# Berechnet die Teiler TSA5511
# Parameter Freq, Dstep , Adresse
sub tsa5511_calc($ $ $) {
my $freq	= $_[0];
my $dstep	= $_[1];
my $adresse	= $_[2];
my $ret;
 if($SERVER) {
  $command = $C_TSA5511_CALC.":".$freq.":".$dstep.":".$adresse.":";
  $ret = send2server($command);
 } else {
   if (($I2C_INIT) && ($TSA5511{$adresse} != -1)) { 
    $ret = i2c_ser::tsa5511_calc($TSA5511{$adresse},$freq,$dstep); 
   } else { $ret = -1; }
  } 
 return $ret;
}

# Sendet die Teiler TSA5511
# Parameter Adresse
sub tsa5511_send($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_TSA5511_SEND.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($TSA5511{$adresse} != -1)) { 
     $ret = i2c_ser::tsa5511_send($TSA5511{$adresse},$adresse); 
    } else { $ret = -1; }
  }  
 return $ret;
}

# Liest StatusByte von PLL
# Parameter Adresse
sub tsa5511_read($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_TSA5511_READ.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($TSA5511{$adresse} != -1)) { 
     $ret = i2c_ser::tsa5511_read($adresse); 
    } else { $ret = -1; }
  }  
 return $ret;
}


# Lscht den Pointer auf TSA5511
# Parameter Adresse
sub tsa5511_delete($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_TSA5511_DELETE.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($TSA5511{$adresse} != -1)) {
     i2c_ser::delete5511($TSA5511{$adresse});
     $TSA5511{$adresse} = -1;
     $ret = 1;
    } else { $ret = -1; }
   }
 return $ret;
}

#------------------------------------------------------------------------------------#
#-----------------------------  Funktionen PLL TSA5523  -----------------------------#
#------------------------------------------------------------------------------------#

# Gibt die def. Konstanten im Array zurck
sub tsa5523_const() {
 return i2c_ser::tsa5523_const();
}

# Initialisiert TSA5523
# Parameter Mode , Adresse
sub tsa5523_init($ $) {
my $mode	= $_[0];
my $adresse	= $_[1];
my $ret;
 if($SERVER) {
  $command = $C_TSA5523_INIT.":".$mode.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($TSA5523{$adresse} == -1)) { # Pointer nur einmal anlegen
     $TSA5523{$adresse} =i2c_ser::create5523(); 
     i2c_ser::tsa5523_init($TSA5523{$adresse},$mode); 
     $ret = 1;
    } else { $ret = -1; }
   } 
 return $ret;
}

# Berechnet die Teiler TSA5523
# Parameter Freq, Dstep , Adresse
sub tsa5523_calc($ $ $) {
my $freq	= $_[0];
my $dstep	= $_[1];
my $adresse	= $_[2];
my $ret;
 if($SERVER) {
  $command = $C_TSA5523_CALC.":".$freq.":".$dstep.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($TSA5523{$adresse} != -1)) { 
     $ret = i2c_ser::tsa5523_calc($TSA5523{$adresse},$freq,$dstep); 
    } else { $ret = -1; }
   } 
 return $ret;
}

# Sendet die Teiler TSA5523
# Parameter Adresse
sub tsa5523_send($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_TSA5523_SEND.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($TSA5523{$adresse} != -1)) { 
     $ret = i2c_ser::tsa5523_send($TSA5523{$adresse},$adresse); 
    } else { $ret = -1; }
   } 
 return $ret;
}

# Liest StatusByte von PLL
# Parameter Adresse
sub tsa5523_read($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_TSA5523_READ.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($TSA5523{$adresse} != -1)) { 
     $ret = i2c_ser::tsa5523_read($adresse); 
    } else { $ret = -1; }
   } 
 return $ret;
}

# Lscht den Pointer auf TSA5523
# Parameter Adresse
sub tsa5523_delete($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_TSA5523_DELETE.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($TSA5523{$adresse} != -1)) {
     i2c_ser::delete5523($TSA5523{$adresse});
     $TSA5523{$adresse} = -1;
     $ret = 1;
    }  else { $ret = -1; }
  } 
 return $ret;
}

#------------------------------------------------------------------------------------#
#-----------------------------  Funktionen PLL SDA3302  -----------------------------#
#------------------------------------------------------------------------------------#

# Gibt die def. Konstanten im Array zurck
sub sda3302_const() {
 return i2c_ser::sda3302_const();
}

# Initialisiert SDA3302
# Parameter Mode , Adresse
sub sda3302_init($ $) {
my $mode	= $_[0];
my $adresse	= $_[1];
my $ret;
 if($SERVER) {
  $command = $C_SDA3302_INIT.":".$mode.":".$adresse.":";
  $ret = send2server($command);
 } else {
     if (($I2C_INIT) && ($SDA3302{$adresse} == -1)) { # Pointer nur einmal anlegen
      $SDA3302{$adresse} =i2c_ser::create3302(); 
      i2c_ser::sda3302_init($SDA3302{$adresse},$mode); 
      $ret = 1;
     } else { $ret = -1; }
   } 
 return $ret;
}

# Berechnet die TeilerSDA3302
# Parameter Freq, Dstep , Adresse
sub sda3302_calc($ $ $) {
my $freq	= $_[0];
my $dstep	= $_[1];
my $adresse	= $_[2];
my $ret;
 if($SERVER) {
  $command = $C_SDA3302_CALC.":".$freq.":".$dstep.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($SDA3302{$adresse} != -1)) { 
     $ret = i2c_ser::sda3302_calc($SDA3302{$adresse},$freq,$dstep); 
    } else { $ret = -1; }
  } 
 return $ret;
}

# Sendet die Teiler SDA3302
# Parameter Adresse
sub sda3302_send($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_SDA3302_SEND.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($SDA3302{$adresse} != -1)) { 
     $ret = i2c_ser::sda3302_send($SDA3302{$adresse},$adresse); 
    } else { $ret = -1; }
   } 
 return $ret;
}


# Lscht den Pointer auf SDA3302
# Parameter Adresse
sub sda3302_delete($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_SDA3302_DELETE.":".$adresse.":";
  $ret = send2server($command);
 } else {
     if (($I2C_INIT) && ($SDA3302{$adresse} != -1)) {
      i2c_ser::delete3302($SDA3302{$adresse});
      $SDA3302{$adresse} = -1;
      $ret = 1;
     } else { $ret = -1; }
  } 
 return $ret;
}

#------------------------------------------------------------------------------------#
#-----------------------------  Funktionen LCD-Display  -----------------------------#
#------------------------------------------------------------------------------------#

# Initialisiert das Display
# Parameter Adresse
sub lcd_init($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_LCD_INIT.":".$adresse.":";
  $ret = send2server($command);
 }else { $ret=i2c_ser::lcd_init($adresse); } 
 return $ret;
}

# Schaltet Licht ein/aus
# Parameter Ein/Aus, 0/1, Adresse
sub lcd_backlight($ $) {
my $onoff	= $_[0];
my $adresse	= $_[1];
my $ret;
 if($SERVER) {
  $command = $C_LCD_BACKLIGHT.":".$onoff.":".$adresse.":";
  $ret = send2server($command);
 } else { $ret = i2c_ser::lcd_backlight($onoff,$adresse); } 
 return $ret;
}

# Commando zum Display Schicken 
# Parameter cmd, Adresse (Offset)
sub lcd_instr($ $) {
my $instr	= $_[0];
my $adresse	= $_[1];
my $ret;
 if($SERVER) {
  $command = $C_LCD_BACKLIGHT.":".$instr.":".$adresse.":";
  $ret = send2server($command);
 } else { $ret = i2c_ser::lcd_instr($instr,$adresse); } 
 return $ret;
}
 
sub lcd_get_adress($) {
my $adresse	= $_[0];
my $ret;
 if($SERVER) {
  $command = $C_LCD_CADR.":".$adresse.":";
  $ret = send2server($command);
 } else { $ret = i2c_ser::lcd_get_adress($adresse); } 
 return $ret;
}
 
# Char zum Display Schicken 
# Parameter char, Adresse (Offset)
sub lcd_wchar($ $) {
my $char	= $_[0];
my $adresse	= $_[1];
my $ret;
 if($SERVER) {
  $command = $C_LCD_WCHAR.":".$char.":".$adresse.":";
  $ret = send2server($command);
 } else { $ret = i2c_ser::lcd_wchar($char,$adresse); } 
 return $ret;
}

# Char vom Display lesen
# Parameter Cursor Pos, Adresse (Offset)
sub lcd_rchar($ $) {
my $cadr	= $_[0];
my $adresse	= $_[1];
my $ret;
 if($SERVER) {
  $command = $C_LCD_RCHAR.":".$cadr.":".$adresse.":";
  $ret = send2server($command);
 } else { $ret = i2c_ser::lcd_rchar($cadr,$adresse);} 
 return $ret;
}

# String zum Display Schicken 
# Parameter str, Adresse (Offset)
sub lcd_write_str($ $) {
my $string	= $_[0];
my $adresse	= $_[1];
my $ret;
 if($SERVER) {
  $command = $C_LCD_WSTR.":".$string.":".$adresse.":";
  $ret = send2server($command);
 } else { $ret = i2c_ser::lcd_write_str($string,$adresse); } 
 return $ret;
}

# String vom Display lesen 
# Parameter len,pos, Adresse (Offset)
sub lcd_read_str($ $ $) {
my $len		= $_[0];
my $pos		= $_[1];
my $adresse	= $_[2];
my $ret;
 if($SERVER) {
  $command = $C_LCD_RSTR.":".$len.":".$pos.":".$adresse.":";
  $ret = send2server($command);
 } else { $ret = i2c_ser::lcd_read_str($len,$pos,$adresse); } 
 return $ret;
}
 
#------------------------------------------------------------------------------------#
#---------------------------  Funktionen PCF8574 8Bit-IO ----------------------------#
#------------------------------------------------------------------------------------#

# Gibt die def. Konstanten im Array zurck
sub pcf8574_const() {
 return i2c_ser::pcf8574_const();
}

sub pcf8574_tx($ $) {
my $data	= $_[0];
my $adresse	= $_[1];
my $ret;

 if($SERVER) {
  $command = $C_PCF8574_TX.":".$data.":".$adresse.":";
  $ret = send2server($command);
 } else { $ret=i2c_ser::iic_tx_pcf8574($data,$adresse); }
 return $ret;
}

sub pcf8574_rx($) {
my $adresse	= $_[0];
my $ret;

 if($SERVER) {
  $command = $C_PCF8574_RX.":".$adresse.":";
  $ret = send2server($command);
 } else { $ret=i2c_ser::iic_rx_pcf8574($adresse); }
 return $ret;
}

#------------------------------------------------------------------------------------#
#---------------------------  Funktionen PCF8591 AD/DA   ----------------------------#
#------------------------------------------------------------------------------------#

# Gibt die def. Konstanten im Array zurck
sub pcf8591_const() {
 return i2c_ser::pcf8591_const();
}

# Initialisiert PCF8591
# Parameter Mode , REFUB
sub pcf8591_init($ $ $) {
my $mode	= $_[0];
my $refub	= $_[1];
my $adresse	= $_[2];
my $ret;

 if($SERVER) {
  $command = $C_PCF8591_INIT.":".$mode.":".$refub.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($PCF{$adresse} == -1)) { # Pointer nur einmal anlegen
     $PCF{$adresse} =i2c_ser::create8591(); 
     i2c_ser::pcf8591_init($PCF{$adresse},$mode,$refub); 
     $ret = 1;
    } else { $ret = -1; }
  }
 return $ret;
}

# Liest einen Kanal aus 
# Parameter Channel, PCF-Adresse
sub pcf8591_read($ $) {
my $channel	= $_[0];
my $adresse	= $_[1];
my $ret;
 
 if($SERVER) {
  $command = $C_PCF8591_READ.":".$channel.":".$adresse.":";
  $ret = send2server($command);
 } else { 
  if (($I2C_INIT) && ($PCF{$adresse} != -1)) { # Pointer nur einmal anlegen
   $ret = i2c_ser::pcf8591_readchan($PCF{$adresse},$channel,$adresse);
  } else { $ret = -1; }
 }
 return $ret;
}

# Liest alle 4 Kanle aus 
# Parameter PCF-Adresse
sub pcf8591_read4chan($) {
my $adresse	= $_[0];
my $ret;

 if($SERVER) {
  $command = $C_PCF8591_READ4.":".$adresse.":";
  $ret = send2server($command);
 } else {  
   if (($I2C_INIT) && ($PCF{$adresse} != -1)) {
    my ($c0,$c1,$c2,$c3) = i2c_ser::pcf8591_read4chan($PCF{$adresse},$adresse);
    $ret = join(':',$c0,$c1,$c2,$c3);
   } else { $ret = -1; }
 } 
 return $ret;
}

# Berechnet fr $channel den Wert in V
# Parameter Channel, PCF-Adresse
sub pcf8591_aout($ $) {
my $channel	= $_[0];
my $adresse	= $_[1];
my $ret;

 if($SERVER) {
  $command = $C_PCF8591_AOUT.":".$channel.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($PCF{$adresse} != -1)) {
     $ret=i2c_ser::pcf8591_aout($PCF{$adresse},$channel);
    } else { $ret = -1; }
   } 
 return $ret;
}


# Setzt den Ausgang DA
# Parameter UB, Adresse
sub pcf8591_setda($ $) {
my $ub		= $_[0];
my $adresse	= $_[1];
my $ret;

 if($SERVER) {
  $command = $C_PCF8591_SETDA.":".$ub.":".$adresse.":";
  $ret = send2server($command);
 } else {
    if (($I2C_INIT) && ($PCF{$adresse} != -1)) {
      $ret=i2c_ser::pcf8591_setda($PCF{$adresse},$ub,$adresse);
    } else { $ret = -1; }
   }  
 return $ret;
}

# Lscht das Handle auf einen PCF
# Parameter Adresse
sub pcf8591_delete($) {
my $adresse	= $_[0];
my $ret;
 
 if($SERVER) {
  $command = $C_PCF8591_DELETE.":".$adresse.":";
  $ret = send2server($command);
 } else {
   if (($I2C_INIT) && ($PCF{$adresse} != -1)) {
    i2c_ser::delete8591($PCF{$adresse});
    $PCF{$adresse} = -1;
    $ret = 1;
   } else { $ret = -1; }
  }
 return $ret;
}

#------------------------------------------------------------------------------------#
#-----------------------------    Interne Funktionen --------------------------------#
#------------------------------------------------------------------------------------#

# Prft ob der Server luft und startet ihn gegebenenfalls.
# Gibt die PID bzw. -1 zurck falls Server nicht luft.
# Parameter : 1. 0/1 bei 1 Starten falls nicht aktiv
# Socket_Name
# Port  ( INET-Socket, derzeit nicht verwendet)
# I2C-Portdelay
sub server_check($ $ $ $ $) {
 my $start_serv = $_[0];
 my $server_cmd = $_[1];
 my $socket	= $_[2];
 my $port	= $_[3];
 my $portdelay	= $_[4];
 my $ret; 
 my @sd;
 $I2C_PORTDELAY = $portdelay;
 $socket_name 	= $socket;
 $PORT		= $port;
 $command = $C_CHECK.":";
 $ret = send2server($command);
 chomp($ret);
 @sd = split(/ /,$ret);
 if (($start_serv) && ($sd[0] == -1)) {
  # Server Starten $start_serv == 1
  system("$server_cmd -s $socket -p $port -d $portdelay > /dev/null &");
 }
 return $ret;
}


# Beendet den Server 
sub server_quit() {
 my $ret;
 $command = $C_QUIT.":";
 $ret = send2server($command);
 return $ret;
}

# Das Ganze noch ohne grosse Fehlerprfung. 
# Greift ein anderer Prozess gerade auf den Server zu, blockiert sysread 
# bis Daten kommen. An Sich kein Problem, da die Verbindung nach dem Lesen beendet 
# wird. Also, ersteinmal beobachten ob's so funkt ...
sub send2server($) {
my $cmd = $_[0]."\n";
if ($PORT==0){
 socket(SOCK,PF_UNIX,SOCK_STREAM,0); #   or die "socket : $!";;
 if (defined connect(SOCK,sockaddr_un($socket_name))) {
   syswrite (SOCK ,$cmd,length($cmd));
   sysread(SOCK,$cmd,10);
   chomp($cmd);
   close(SOCK); 
 } else  { $cmd = -1; }
} else {
 # Server connecten
  my $remote_host = 'linuxw95';
  my $server = IO::Socket::INET->new(PeerAddr   => $remote_host,
                                     PeerPort      => $PORT,
                                     Proto         => 'tcp',
                                     Type          => SOCK_STREAM); #  || die $@;
  if ($server) {
   syswrite ($server ,$cmd,length($cmd));
   sysread($server,$cmd,10);
   chomp($cmd);
   close($server);
  } else  { $cmd = -1; }   
 }
 return $cmd;
}



1;

__END__

=head1 NAME

i2c_client - Perl Client for i2c_ser devices Ver. 0.1 / 12.09.2001

=head1 SYNOPSIS

	use i2c_client;
	
	Available Functions.
	i2c_client::server_check($ $ $ $) {
 	i2c_client::init($);
	i2c_client::re_init();
	i2c_client::server_quit();

	i2c_client::tsa6057_const();
	i2c_client::tsa6057_init($ $ $);
	i2c_client::tsa6057_calc($ $ $);
	i2c_client::tsa6057_send($);
	i2c_client::tsa6057_delete($);

	i2c_client::tsa5511_const();
	i2c_client::tsa5511_init($ $ $);
	i2c_client::tsa5511_calc($ $ $);
	i2c_client::tsa5511_send($);
	i2c_client::tsa5511_read($);
	i2c_client::tsa5511_delete($);

	i2c_client::tsa5523_const();
	i2c_client::tsa5523_init($ $);	
	i2c_client::tsa5523_calc($ $ $);
	i2c_client::tsa5523_send($);
	i2c_client::tsa5523_read($);	
	i2c_client::tsa5523_delete($);

	i2c_client::sda3302_const();
	i2c_client::sda3302_init($ $);	
	i2c_client::sda3302_calc($ $ $);
	i2c_client::sda3302_send($);
	i2c_client::sda3302_delete($);

	i2c_client::pcf8574_const();
	i2c_client::pcf8574_tx($ $);
	i2c_client::pcf8574_rx($);

	i2c_client::pcf8591_const();
        i2c_client::create8591($);
        i2c_client::pcf8591_init($ $ $);
        i2c_client::pcf8591_readchan($ $)
        i2c_client::pcf8591_read4chan($);
        i2c_client::pcf8591_aout($ $);
        i2c_client::pcf8591_setda($ $);
        i2c_client::delete8591($);

	i2c_client::lcd_init($);
	i2c_client::lcd_instr($ $);
	i2c_client::lcd_wchar($ $);
	i2c_client::lcd_write_str('$ $);
	i2c_client::lcd_get_adress($);
	i2c_client::lcd_backlight($ $);
	i2c_client::lcd_rchar($ $);
	i2c_client::lcd_read_str($ $ $);


=head1 DESCRIPTION

  *** Allgemeine IC-Routinen ***

	i2c_client::server_check($ $ $ $ $) {
        $line = i2c_client::server_check(1,'/opt/i2c/i2c_server.pl',$socket_name,$PORT,$I2C_PORTDELAY);
	# Prft ob der Server luft und startet ihn gegebenenfalls.
	# Gibt die PID bzw. -1 zurck falls Server nicht luft.
	# Parameter : 1. 0/1 bei 1 Starten falls nicht aktiv.

 	i2c_client::init($);
	# Rckgabe Ser-Port bzw. -1 falls Server nicht aktiv, 
	# 0 wenn kein I2C-Interface gefunden wurde.
	# Parameter : 0 Hardware direkt, 1 ber Server.

	i2c_client::re_init();
	# Fhrt ein reinit i2c auf dem Server aus.
	# Rckgabe Ser-Port bzw. -1 falls Server nicht aktiv, 
	# 0 wenn kein I2C-Interface gefunden wurde.

	i2c_client::server_quit();
	# Beendet den Server 

	#------------------------------------------#
	#--  Funktionen PLL TSA6057  --------------#
	#------------------------------------------#

	i2c_client::tsa6057_const();
	#@arr = i2c_client::tsa6057_const();
	#TSA6057 Konst : 196 16 32 0 64 128  
	#($tsa6057_adr,$tsa6057_am,$tsa6057_fm,$tsa6057_R01,$tsa6057_R10,$tsa6057_R25)

	i2c_client::tsa6057_init($ $ $);
	# Raster , Mode, Adresse , Rckgabe Handler
	$tsa6057  = i2c_client::tsa6057_init($tsa6057_R25,$tsa6057_fm,0);

	i2c_client::tsa6057_calc($ $ $);
	# Berechnet die Teiler
	# Parameter Freq,Raster, AdressOffset
	$frequenz = 47.995e6;
	$tsa6057  = i2c_client::tsa6057_calc($frequenz,$tsa6057_R25,0);

	i2c_client::tsa6057_send($);
	# Sendet die Teiler zur PLL
	# Parameter AdressOffset
	$tsa6057  = i2c_client::tsa6057_send(0);

	i2c_client::tsa6057_delete($);
	# Lscht den Pointer 
	# Parameter AdressOffset
	$tsa6057  = i2c_client::tsa6057_delete(0);

	#------------------------------------------#
	#--  Funktionen PLL TSA5511  --------------#
	#------------------------------------------#

	i2c_client::tsa5511_const();
	#@arr = i2c_client::tsa5511_const();
	#Konst : 192 62500 206 238
	#($tsa5511_adr,$tsa5511_step,$tsa5511_PLL,$tsa5511_DIV)

	i2c_client::tsa5511_init($ $ $);
	# Mode, RefFrequenz , Adresse , Rckgabe Handler
	# my $refq = 4.0e6;
	# my $tsa5511  = i2c_client::tsa5511_init($tsa5511_PLL,$refq,$Adresse);

	i2c_client::tsa5511_calc($ $ $);
	# Berechnet die Teiler
	# Parameter Freq,DoppelStep (0/1), AdressOffset
	$frequenz = 439.20e6;
	$tsa5511  = i2c_client::tsa5511_calc($frequenz,0,$Adresse);

	i2c_client::tsa5511_send($);
	# Sendet die Teiler zur PLL
	# Parameter AdressOffset
	$tsa5511  = i2c_client::tsa5511_send($Adresse);

	i2c_client::tsa5511_read($);
	# Liest StatusByte von PLL
	#Parameter AdressOffset
	$tsa5511  = i2c_client::tsa5511_read($Adresse);

	i2c_client::tsa5511_delete($);
	#Lscht den Pointer 
	# Parameter AdressOffset
	$tsa5511  = i2c_client::tsa5511_delete($Adresse);

	#------------------------------------------#
	#---  Funktionen PLL TSA5523 --------------#
	#------------------------------------------#

	i2c_client::tsa5523_const();
	#@arr = i2c_client::tsa5523_const();
	#TSA5523 Konst : 192 50000 37300000 200 238
	#($tsa5523_adr,$tsa5523_step,$tsa5523_zf,$tsa5523_PLL ,$tsa5523_DIV)

	i2c_client::tsa5523_init($ $);	
	# Mode, Adresse , Rckgabe Handler
	my $tsa5523  = i2c_client::tsa5523_init($tsa5523_PLL,0);

	i2c_client::tsa5523_calc($ $ $);
	# Berechnet die Teiler
	# Parameter Freq,DoppelStep (0/1), AdressOffset
	$frequenz = 104.1e6;
	$tsa5523  = i2c_client::tsa5523_calc($frequenz,0,0);

	i2c_client::tsa5523_send($);
	# Parameter AdressOffset
	$tsa5523  = i2c_client::tsa5523_send(0);

	i2c_client::tsa5523_read($);	
	# Liest StatusByte von PLL
	# Parameter AdressOffset
	$tsa5523  = i2c_client::tsa5523_read(0);

	i2c_client::tsa5523_delete($);
	# Lscht den Pointer 
	# Parameter AdressOffset
	$tsa5523  = i2c_client::tsa5523_delete(0);

	#------------------------------------------#
	#---  Funktionen PLL SDA3302 --------------#
	#------------------------------------------#

	i2c_client::sda3302_const();
	# @arr = i2c_client::sda3302_const();
	# Konst : 192 62500 37300000 206 238
	#($sda3302_adr,$sda3302_step,$sda3302_zf,$sda3302_PLL,$sda3302_DIV)

	i2c_client::sda3302_init($ $);	
	# Mode, Adresse , Rckgabe Handler
	$sda3302  = i2c_client::sda3302_init($sda3302_PLL,0);

	i2c_client::sda3302_calc($ $ $);
	# Berechnet die Teiler
	# Parameter Freq,DoppelStep (0/1), AdressOffset
	$sda3302  = i2c_client::sda3302_calc(104.100e6,0,0);

	i2c_client::sda3302_send($);
	# Sendet die Teiler zur PLL
	# Parameter AdressOffset
	$sda3302  = i2c_client::sda3302_send(0);

	i2c_client::sda3302_delete($);
	# Lscht den Pointer 
	# Parameter AdressOffset
	$sda3302  = i2c_client::sda3302_delete(0);

	#------------------------------------------#
	#---   Funktionen PCF8574  ----------------#
	#------------------------------------------#

	i2c_client::pcf8574_const();
	#@arr = i2c_client::pcf8574_const();
	#Konst : 112 113
	#($tx_8574,$rx_8574)

	i2c_client::pcf8574_tx($ $);
	# TX Data-Byte, Adresse 
	# Sendet das Byte 'Data' zum PCF8574.
        # Ist die Adresse <> 112 , muss Adresss_Offste entsprechend gesetz werden.
        # Sonst Adress_Offset=0
	# i2c_client::pcf8574_tx(255,2);

	i2c_client::pcf8574_rx($);
	# RX , Adresse
	$rxbyte  = i2c_client::pcf8574_rx(2);

	#------------------------------------------#
	#---   Funktionen PCF8591  ----------------#
	#------------------------------------------#

	i2c_client::pcf8591_const()
	# @arr = i2c_client::pcf8591_const();
	#($pcf8591_adr,$pcf8591_rx,$pcf8591_c4s,$pcf8591_c3d,$pcf8591_c2s,$pcf8591_c2d) 
	# Konst: 144 145 0 16 32 48 16

	i2c_client::pcf8591_init($ $ $);
	# Mode, REF-UB,Adresse , Rckgabe Handler
	# $pcf  = i2c_client::pcf8591_init($pcf8591_c4s,2.56,0);

        i2c_client::pcf8591_readchan($ $)
	# Kanal , Adresse
	# $line  = i2c_client::pcf8591_read(2,0);

        i2c_client::pcf8591_read4chan($);
	# Adresse
	# $line  = i2c_client::pcf8591_read4chan(0);

        i2c_client::pcf8591_aout($ $);
	# Ub, Adresse
	# $line  = i2c_client::pcf8591_setda(1.0,0);

        i2c_client::pcf8591_setda($ $);
	# Kanal , Adresse
	# $line  = i2c_client::pcf8591_aout(2,0);

        i2c_client::delete8591($);
	#Adresse
	i2c_client::pcf8591_delete(0);
        

	#------------------------------------------#
	#---   Funktionen LCD      ----------------#
	#------------------------------------------#

	# LCD Befehle
	# Cursor Off    = 0x0c
	# Cursor On     = 0x0e
	# Cursor home   = 0x02
	# Cursor blink  = 0x0f
	# Display Clear = 0x01
	# Display Off   = 0x08
	# Adresse setz. = 0x80

	i2c_client::lcd_init($);
	# Adresse (Offset)
	i2c_client::lcd_init(0);

	i2c_client::lcd_instr($ $);
	# Commando zum Display Schicken 
	# Parameter cmd, Adresse (Offset)
	i2c_client::lcd_instr(0x0f,0);

	i2c_client::lcd_wchar($ $);
	# Char zum Display Schicken 
	# Parameter char, Adresse (Offset)
	i2c_client::lcd_wchar(ord('H'),0);

	i2c_client::lcd_write_str('$ $);
	# String zum Display Schicken 
	# Parameter String, Adresse (Offset)
	i2c_client::lcd_write_str(' Display',0);

	i2c_client::lcd_get_adress($);
	# Cursor Adresse holen
	#Parameter Adresse (Offset)  
	i2c_client::lcd_get_adress(0);

	i2c_client::lcd_backlight($ $);
	#Licht Ein/Aus Adresse (Offset)
	i2c_client::lcd_backlight(1,0);

	i2c_client::lcd_rchar($ $);
	#Zeichen an Cursor Adresse lesen
	# Parameter CursorPosition , Adresse (Offset)  
	i2c_client::lcd_rchar(0,0);

	i2c_client::lcd_read_str($ $ $);
	# Liest String mit Len ab Pos vom DIsplay
	# Parameter Lnge, CursorPosition , Adresse (Offset)  
	$line=i2c_client::lcd_read_str(5,0,0);


=head1 AUTHOR
IngoGerlach@welfen-netz.com (DH1AAD) 14.09.2001

=head1 SEE ALSO

perl(1).

=cut
