#!/usr/bin/perl -w 
 
# I2C-Server, max. 1 Connection
# (c) DH1AAD / Ingo Gerlach  
# 
# Im Gegensatz zu den "normalen" Servern wird mit diesem hier erreicht 
# das max. nur 1 Prozess auf den I2C-Bus zugreift. Es wird damit verhindert
# das es zu "dead-locks" kommt wenn z.B. ein Proz. gerade Daten liest whrend
# der andere Prozess gerade ein INIT-I2C durchfhren will.
# Nachteil, die gesamt Last des Prozessors steigt natrlich und die
# max. Geschwindigkeit ist etwas verringert. Beim Scannen ca. 27 Kanle / sec
# gegenber 33 Kanle / sec. wenn der Bus direkt angesteuert wird.  
# 
# To DO 
#
# 0.4a	05.03.2002	LogFile Size beim Start prfen
# 0.4	14.09.2001	Client und Test-Client OK. Funktionen werden nun als Hash-Referenz bergeben
# 0.3	08.09.2001	Funktionen definieren, paralell i2c_client.pm schreiben
# 0.2	03.09.2001	Grundstzlich funkt's nu.... :)
# 0.1 	02.09.2001	Start .... :(, 
#


use strict 'vars'; 
use i2c_ser; 
use IO::Socket;
use Net::hostent;              # for OO version of gethostbyaddr
use Time::Local;
 
use vars qw($opt_s $opt_p $opt_d $opt_l $opt_b $opt_k);
use Getopt::Std;


sub get_options();

my $client;
my $server;
my $socket_name 	= '/tmp/i2c_device';
my $proto		= getprotobyname('tcp');
my $PORT;

my $Debug		= 0;
my $command="0";


# Hardware 
my $I2C_INIT    	= 0;			# True , wenn Init OK  
my $I2C_PORTDELAY	= 10;			# Hardware  
my $I2C_TYPE		= "SER";		# Hardware  Port, nur Info
my $I2C_port 		= 0;			# Port, automatisch suchen
my $I2C_LOG		= "i2c_server.log"; 	# LogFile
my $I2C_LOG_SIZE	= 1000000;
my $I2C_DEBUG		= 0;			# Debug Off

my @sd;
#------------------------------------------------------------------------------------#
#---------------------------    Funktions Konstanten  -------------------------------#
#------------------------------------------------------------------------------------#

my $C_QUIT		= "0000";
my $C_CHECK		= "0001";
my $C_INIT		= "1000";
my $C_REINIT		= "1001";
my $C_PCF8591_INIT	= "2000";
my $C_PCF8591_READ	= "2001";
my $C_PCF8591_READ4	= "2002";
my $C_PCF8591_AOUT	= "2003";
my $C_PCF8591_SETDA	= "2004";
my $C_PCF8591_DELETE	= "2005";
my $C_PCF8574_TX	= "3000";
my $C_PCF8574_RX	= "3001";
my $C_LCD_INIT		= "3100";
my $C_LCD_BACKLIGHT	= "3101"; 
my $C_LCD_INSTR		= "3102"; 
my $C_LCD_CADR		= "3103";  
my $C_LCD_WCHAR		= "3104";  
my $C_LCD_WSTR		= "3105";   
my $C_LCD_RCHAR		= "3106";  
my $C_LCD_RSTR		= "3107";   
my $C_SDA3302_INIT	= "4000";   
my $C_SDA3302_CALC	= "4001";   
my $C_SDA3302_SEND	= "4002";   
my $C_SDA3302_DELETE	= "4003";    
my $C_TSA5523_INIT	= "5000";   
my $C_TSA5523_CALC	= "5001";   
my $C_TSA5523_SEND	= "5002";   
my $C_TSA5523_READ	= "5003";   
my $C_TSA5523_DELETE	= "5004";    
my $C_TSA5511_INIT	= "6000";   
my $C_TSA5511_CALC	= "6001";   
my $C_TSA5511_SEND	= "6002";   
my $C_TSA5511_READ	= "6003";   
my $C_TSA5511_DELETE	= "6004";    
my $C_TSA6057_INIT	= "7000";   
my $C_TSA6057_CALC	= "7001";   
my $C_TSA6057_SEND	= "7002";   
my $C_TSA6057_DELETE	= "7003";    

my %CMD			= ($C_QUIT		=>sub {print $client "1\n";exit(1);},
			   $C_CHECK		=>sub {print $client "$$";},
			   $C_INIT		=>sub {return $I2C_port;},
			   $C_REINIT		=>sub {init_port(); return $I2C_port;},
			   #-------  TSA5511 -------#
			   $C_TSA5511_INIT	=>sub {return tsa_5511_init($sd[1],$sd[2],$sd[3]); },
			   $C_TSA5511_CALC	=>sub {return tsa_5511_calc($sd[1],$sd[2],$sd[3]); },
			   $C_TSA5511_SEND	=>sub {return tsa_5511_send($sd[1]); },
			   $C_TSA5511_READ	=>sub {return tsa_5511_read($sd[1]); },
		           $C_TSA5511_DELETE	=>sub {return tsa_5511_delete($sd[1]);},
			   #-------  PCF8591 -------#
			   $C_PCF8591_INIT	=>sub {return pcf_8591_init($sd[1],$sd[2],$sd[3]); },
			   $C_PCF8591_READ	=>sub {return pcf_8591_read($sd[1],$sd[2]); },
			   $C_PCF8591_READ4	=>sub {return pcf_8591_read4($sd[1]); },
			   $C_PCF8591_AOUT	=>sub {return pcf_8591_aout($sd[1],$sd[2]); },
			   $C_PCF8591_SETDA	=>sub {return pcf_8591_setda($sd[1],$sd[2]); },
			   $C_PCF8591_DELETE	=>sub {return pcf_8591_delete($sd[1]); },
		           #------  PCF8574 --------#
			   $C_PCF8574_TX	=>sub {return pcf_8574_tx($sd[1],$sd[2]); },
                           $C_PCF8574_RX	=>sub {return pcf_8574_rx($sd[1]); },
			   #------  SDA3302 --------#
			   $C_SDA3302_INIT	=>sub {return sda_3302_init($sd[1],$sd[2]); },
			   $C_SDA3302_CALC	=>sub {return sda_3302_calc($sd[1],$sd[2],$sd[3]); },
			   $C_SDA3302_SEND	=>sub {return sda_3302_send($sd[1]); },
                           $C_SDA3302_DELETE	=>sub {return sda_3302_delete($sd[1]); },
			   #------  TSA5523 --------#
			   $C_TSA5523_INIT	=>sub { return tsa_5523_init($sd[1],$sd[2]); },
                           $C_TSA5523_CALC	=>sub { return tsa_5523_calc($sd[1],$sd[2],$sd[3]); },
                           $C_TSA5523_SEND	=>sub { return tsa_5523_send($sd[1]); },
                           $C_TSA5523_READ	=>sub { return tsa_5523_read($sd[1]); },
                           $C_TSA5523_DELETE	=>sub { return tsa_5523_delete($sd[1]); },
			   #------  TSA6057 --------#
			   $C_TSA6057_INIT	=>sub { return tsa_6057_init($sd[1],$sd[2],$sd[3]); },
                           $C_TSA6057_CALC	=>sub { return tsa_6057_calc($sd[1],$sd[2],$sd[3]); },
                           $C_TSA6057_SEND	=>sub { return tsa_6057_send($sd[1]); },
                           $C_TSA6057_DELETE	=>sub { return tsa_6057_delete($sd[1]); },
		           #------  L C D   --------#
			   $C_LCD_INIT		=>sub { return lcd_init($sd[1]); },
                           $C_LCD_BACKLIGHT	=>sub { return lcd_backlight($sd[1],$sd[2]); }, 
                           $C_LCD_INSTR		=>sub { return lcd_backlight($sd[1],$sd[2]); }, 
                           $C_LCD_CADR		=>sub { return lcd_get_adress($sd[1]); }, 
                           $C_LCD_WCHAR		=>sub { return lcd_wchar($sd[1],$sd[2]); }, 
                           $C_LCD_WSTR		=>sub { return lcd_write_str($sd[1],$sd[2]); }, 
                           $C_LCD_RCHAR		=>sub { return lcd_rchar($sd[1],$sd[2]); }, 
                           $C_LCD_RSTR		=>sub { return lcd_read_str($sd[1],$sd[2],$sd[3]); } 
			   );
 
 
# PCF8591 , Hash auf Handles
my %PCF			= ("0"=>-1,"2"=>-1,"4"=>-1,"6"=>-1,"8"=>-1,"10"=>-1,"12"=>-1,"14"=>-1);
my %SDA3302		= ("0"=>-1,"2"=>-1,"4"=>-1);
my %TSA5523		= ("0"=>-1,"2"=>-1,"4"=>-1,"6"=>-1);
my %TSA5511		= ("0"=>-1,"2"=>-1,"4"=>-1,"6"=>-1);
my %TSA6057		= ("0"=>-1,"2"=>-1);

# Func forwarding	
sub logmsg(@);

# Start

 # Config einlesen
 # command line params
 get_options(); 

 #Main 
 check_log_size($I2C_LOG,$I2C_LOG_SIZE);
 # open (STDERR,">>$I2C_LOG") || die "can't open Logfile $I2C_LOG. \n"; 
 logmsg("debug $I2C_DEBUG");
 $I2C_port = init_port();

#------------------------------------------------------------------------------------#
#------------------------------    Server Starten   ---------------------------------#
#------------------------------------------------------------------------------------#

 
 # if($PORT==0) { # Unix-Socket
  unlink $socket_name;
  $server = IO::Socket::UNIX->new(Local	 	=> $socket_name,
  				  Type		=> SOCK_STREAM,
 				  Reuse		=> 1,
 				  Listen	=> 10) || die $@;
  logmsg "I2C-Server gestartet. (Unix-Domain) ". $socket_name;

 # }
 # else { 				  
 # $server = IO::Socket::INET->new(Proto         => 'tcp',
 #                                 LocalPort 	=> $PORT,
 #                                 Listen    	=> SOMAXCONN,
 #                                 Reuse     	=> 1) || die $@;;
 # logmsg "I2C-Server gestartet. (Inet) Port: ".$PORT;
 #}                                 
				 
 
 while($client = $server->accept()) {
  $client->autoflush(1);
  #if($Debug) { print $client "Welcome to I2C-Server 0.1 ($0); type help for command list.\n"; }
  #if($Debug) { print $client "(Debug-Mode) Command? "; }
  while ( <$client>) {
    next unless /\S/;                   # blank line
    if($Debug) { print "CMD: $_"; }
    print $client &check_command($_) ; 
   }
  close $client;
 } # while

 close STDERR; 
#------------------------------------------------------------------------------------#
#-----------------------------    Command Dispatcher  -------------------------------#
#------------------------------------------------------------------------------------#


sub check_command($) {
my $line = $_[0];
my $param_cnt;

 $line =~ s/([^\x0a])\x0d\x0a/$1 /g;
 chomp($line);
 @sd = split(/:/, $line); 	# Splitten
 $param_cnt = (@sd)-2;
 if ($I2C_DEBUG) {logmsg("CMD $line , Cnt : $param_cnt , CMD $sd[0]"); }
 
 if ($CMD{$sd[0]}) {
  return $CMD{$sd[0]}->();
 } else { return -1; }
}

#------------------------------------------------------------------------------------#
#------------------------    Allgemeine I2C- Funktionen  ----------------------------#
#------------------------------------------------------------------------------------#

# Prueft ob I2C - Interface angeschlossen ist. 
sub init_port() { 
my $tport = $I2C_port;	# 
my $tcnt  = 5; 		# max. 5 Versuche 

 i2c_ser::set_port_delay($I2C_PORTDELAY);  # Port Delay 
 # Interface suchen 
 while($tcnt) { 
  $I2C_port = i2c_ser::init_iic($tport); 
  sleep(1);
  $tcnt--;
  if ($I2C_port > 0) { $tcnt = 0; }
 }
 if ($I2C_port > 0) { 
  my $temp = sprintf("Server:I2C-Interface found on Port 0x%0xh",$I2C_port); 
  logmsg($temp);
 } else { 
  logmsg("I2C-Interface not found."); 
 } 
 $I2C_INIT = ( $I2C_port > 0 ? 1 : 0);
 return $I2C_port; 
} 

#------------------------------------------------------------------------------------#
#----------------------------   Funktionen TSA6057 PLL   ----------------------------#
#------------------------------------------------------------------------------------#

# PLL Datenpointer anlegen 
sub tsa_6057_init() {
my $raster	= $_[0];
my $mode 	= $_[1];
my $adresse	= $_[2];
my $ret		= 1;

 if (($I2C_INIT) && ($TSA6057{$adresse} == -1)) { # Pointer nur einmal anlegen
  $TSA6057{$adresse} =i2c_ser::create6057(); 
  i2c_ser::tsa6057_init($TSA6057{$adresse},$raster,$mode); 
  $ret = 1;
 }
 return $ret; 
}

# PLL Daten Berechnen
sub tsa_6057_calc() {
my $freq	= $_[0];
my $raster	= $_[1];
my $adresse	= $_[2];
my $ret		= 0;

 if (($I2C_INIT) && ($TSA6057{$adresse} != -1)) { 
  $ret = i2c_ser::tsa6057_calc($TSA6057{$adresse},$freq,$raster); 
 }
 return $ret; 
}

# PLL Datenpointer senden
sub tsa_6057_send() {
my $adresse	= $_[0];
my $ret		= 0;

 if (($I2C_INIT) && ($TSA6057{$adresse} != -1)) { 
  $ret = i2c_ser::tsa6057_send($TSA6057{$adresse},$adresse); 
 }
 return $ret; 
}

sub tsa_6057_delete() {
my $adresse     = $_[0];
 if (($I2C_INIT) && ($TSA6057{$adresse} != -1)) {
  i2c_ser::delete6057($TSA6057{$adresse});
  $TSA6057{$adresse} = -1;
  return 1;
 } 
 return -1;
}

#------------------------------------------------------------------------------------#
#----------------------------   Funktionen TSA5511 PLL   ----------------------------#
#------------------------------------------------------------------------------------#

# PLL Datenpointer anlegen 
sub tsa_5511_init() {
my $mode 	= $_[0];
my $refquarz	= $_[1];
my $adresse	= $_[2];
my $ret		= -1;

 if (($I2C_INIT) && ($TSA5511{$adresse} == -1)) { # Pointer nur einmal anlegen
  $TSA5511{$adresse} =i2c_ser::create5511(); 
  i2c_ser::tsa5511_init($TSA5511{$adresse},$mode,$refquarz); 
  $ret = 1;
 }
 return $ret; 
}

# PLL Daten Berechnen
sub tsa_5511_calc() {
my $freq	= $_[0];
my $dstep	= $_[1];
my $adresse	= $_[2];
my $ret		= 0;

 if (($I2C_INIT) && ($TSA5511{$adresse} != -1)) { 
  $ret = i2c_ser::tsa5511_calc($TSA5511{$adresse},$freq,$dstep); 
 }
 return $ret; 
}

# PLL Datenpointer senden
sub tsa_5511_send() {
my $adresse	= $_[0];
my $ret		= 0;

 if (($I2C_INIT) && ($TSA5511{$adresse} != -1)) { 
  $ret = i2c_ser::tsa5511_send($TSA5511{$adresse},$adresse); 
 }
 return $ret; 
}

# StatusByte lesen
sub tsa_5511_read() {
my $adresse	= $_[0];
my $ret		= 1;

 if (($I2C_INIT) && ($TSA5511{$adresse} != -1)) { 
  $ret = i2c_ser::tsa5511_read($adresse); 
 }
 return $ret; 
}


sub tsa_5511_delete() {
my $adresse     = $_[0];
 if (($I2C_INIT) && ($TSA5511{$adresse} != -1)) {
  i2c_ser::delete5511($TSA5511{$adresse});
  $TSA5511{$adresse} = -1;
  return 1;
 } 
 return -1;
}


#------------------------------------------------------------------------------------#
#----------------------------   Funktionen TSA5523 PLL   ----------------------------#
#------------------------------------------------------------------------------------#

# PLL Datenpointer anlegen 
sub tsa_5523_init() {
my $mode 	= $_[0];
my $adresse	= $_[1];
my $ret		= 1;

 if (($I2C_INIT) && ($TSA5523{$adresse} == -1)) { # Pointer nur einmal anlegen
  $TSA5523{$adresse} =i2c_ser::create5523(); 
  i2c_ser::tsa5523_init($TSA5523{$adresse},$mode); 
  $ret = 1;
 }
 return $ret; 
}

# PLL Daten Berechnen
sub tsa_5523_calc() {
my $freq	= $_[0];
my $dstep	= $_[1];
my $adresse	= $_[2];
my $ret		= 0;

 if (($I2C_INIT) && ($TSA5523{$adresse} != -1)) { 
  $ret = i2c_ser::tsa5523_calc($TSA5523{$adresse},$freq,$dstep); 
 }
 return $ret; 
}

# PLL Datenpointer senden
sub tsa_5523_send() {
my $adresse	= $_[0];
my $ret		= 0;

 if (($I2C_INIT) && ($TSA5523{$adresse} != -1)) { 
  $ret = i2c_ser::tsa5523_send($TSA5523{$adresse},$adresse); 
 }
 return $ret; 
}

# StatusByte lesen
sub tsa_5523_read() {
my $adresse	= $_[0];
my $ret		= 1;

 if (($I2C_INIT) && ($TSA5523{$adresse} != -1)) { 
  $ret = i2c_ser::tsa5523_read($adresse); 
 }
 return $ret; 
}


sub tsa_5523_delete() {
my $adresse     = $_[0];
 if (($I2C_INIT) && ($TSA5523{$adresse} != -1)) {
  i2c_ser::delete5523($TSA5523{$adresse});
  $TSA5523{$adresse} = -1;
  return 1;
 } 
 return -1;
}

#------------------------------------------------------------------------------------#
#----------------------------   Funktionen SDA3302 PLL   ----------------------------#
#------------------------------------------------------------------------------------#

# PLL Datenpointer anlegen Mode 206 = PLL
sub sda_3302_init() {
my $mode 	= $_[0];
my $adresse	= $_[1];
my $ret		= 1;

 if (($I2C_INIT) && ($SDA3302{$adresse} == -1)) { # Pointer nur einmal anlegen
  $SDA3302{$adresse} =i2c_ser::create3302(); 
  i2c_ser::sda3302_init($SDA3302{$adresse},$mode); 
  $ret = 1;
 }
 return $ret; 
}

# PLL Daten Berechnen
sub sda_3302_calc() {
my $freq	= $_[0];
my $dstep	= $_[1];
my $adresse	= $_[2];
my $ret		= 0;

 if (($I2C_INIT) && ($SDA3302{$adresse} != -1)) { 
  $ret = i2c_ser::sda3302_calc($SDA3302{$adresse},$freq,$dstep); 
 }
 return $ret; 
}

# PLL Datenpointer senden
sub sda_3302_send() {
my $adresse	= $_[0];
my $ret		= 0;

 if (($I2C_INIT) && ($SDA3302{$adresse} != -1)) { 
  $ret = i2c_ser::sda3302_send($SDA3302{$adresse},$adresse); 
 }
 return $ret; 
}

sub sda_3302_delete() {
my $adresse     = $_[0];
 if (($I2C_INIT) && ($SDA3302{$adresse} != -1)) {
  i2c_ser::delete3302($SDA3302{$adresse});
  $SDA3302{$adresse} = -1;
  return 1;
 } 
 return -1;
}

#------------------------------------------------------------------------------------#
#-----------------------------  Funktionen LCD-Display  -----------------------------#
#------------------------------------------------------------------------------------#
sub lcd_init() {
my $adresse	= $_[0];
 return i2c_ser::lcd_init($adresse);
}

sub lcd_backlight() {
my $onoff	= $_[0];
my $adresse	= $_[1];
 return i2c_ser::lcd_backlight($onoff,$adresse);
}

sub lcd_instr() {
my $instr	= $_[0];
my $adresse	= $_[1];
 return i2c_ser::lcd_instr($instr,$adresse);
}

sub lcd_get_adress() {
my $adresse	= $_[0];
 return i2c_ser::lcd_get_adress($adresse);
}

sub lcd_wchar() {
my $char	= $_[0];
my $adresse	= $_[1];
 return i2c_ser::lcd_wchar($char,$adresse);
}

sub lcd_write_str() {
my $string	= $_[0];
my $adresse	= $_[1];
 return i2c_ser::lcd_write_str($string,$adresse);
}

sub lcd_rchar() {
my $cadr	= $_[0];
my $adresse	= $_[1];
 return i2c_ser::lcd_rchar($cadr,$adresse);
}

sub lcd_read_str() {
my $len		= $_[0];
my $pos		= $_[1];
my $adresse	= $_[2];
 return i2c_ser::lcd_read_str($len,$pos,$adresse);
}

#------------------------------------------------------------------------------------#
#---------------------------  Funktionen PCF8574 8Bit-IO ----------------------------#
#------------------------------------------------------------------------------------#

sub pcf_8574_tx() {
my $data	= $_[0];
my $adresse	= $_[1];
 return i2c_ser::iic_tx_pcf8574($data,$adresse);
}

sub pcf_8574_rx() {
my $adresse	= $_[0];
 return i2c_ser::iic_rx_pcf8574($adresse);
}

#------------------------------------------------------------------------------------#
#---------------------------  Funktionen PCF8591 AD/DA   ----------------------------#
#------------------------------------------------------------------------------------#

# AD/DA Wandler PCF8591 Datenpointer anlegen Mode 0 = 4x Eingang 
sub pcf_8591_init() {
my $mode 	= $_[0];
my $pcf_refub   = $_[1];
my $adresse	= $_[2];
my $ret		= 1;
 if (($I2C_INIT) && ($PCF{$adresse} == -1)) { # Pointer nur einmal anlegen
  $PCF{$adresse} =i2c_ser::create8591(); 
  i2c_ser::pcf8591_init($PCF{$adresse},$mode,$pcf_refub); 
  $ret = 1;
 }
 return $ret; 
}

sub pcf_8591_read() {
my $PCF_CHANNEL = $_[0];
my $adresse     = $_[1];
 if (($I2C_INIT) && ($PCF{$adresse} != -1)) {
   return i2c_ser::pcf8591_readchan($PCF{$adresse},$PCF_CHANNEL,$adresse);
 } 
 return -1;
}
 
sub pcf_8591_read4() {
my $adresse     = $_[0];
my $line ; 
 if (($I2C_INIT) && ($PCF{$adresse} != -1)) {
  $line = i2c_ser::pcf8591_read4chan($PCF{$adresse},$adresse);
  # print "Line $line \n";
  my ($c0,$c1,$c2,$c3) = i2c_ser::pcf8591_read4chan($PCF{$adresse},$adresse);
  return join(':',$c0,$c1,$c2,$c3);
 } 
 return -1;
}
 
sub pcf_8591_aout() {
my $PCF_CHANNEL = $_[0];
my $adresse     = $_[1];
 if (($I2C_INIT) && ($PCF{$adresse} != -1)) {
  return i2c_ser::pcf8591_aout($PCF{$adresse},$PCF_CHANNEL);
 } 
 return -1;
}

sub pcf_8591_setda() {
my $ub		= $_[0];
my $adresse     = $_[1];
 if (($I2C_INIT) && ($PCF{$adresse} != -1)) {
  return i2c_ser::pcf8591_setda($PCF{$adresse},$ub,$adresse);
 } 
 return -1;
}

sub pcf_8591_delete() {
my $adresse     = $_[0];
 if (($I2C_INIT) && ($PCF{$adresse} != -1)) {
  i2c_ser::delete8591($PCF{$adresse});
  $PCF{$adresse} = -1;
  return 1;
 } 
 return -1;
}

#------------------------------------------------------------------------------------#
#-----------------------------    Interne Funktionen --------------------------------#
#------------------------------------------------------------------------------------#

sub get_options() {
# Options ???
 $opt_s = "";
 $opt_p = "";
 $opt_d = "";
 $opt_l = "";
 $opt_b = "";
 $opt_k = "";
 getopts('s:p:d:l:b:k:');
 if ($opt_s ne "") { $socket_name=$opt_s;}
 if ($opt_p ne "") { $PORT=$opt_p;}
 if ($opt_d ne "") { $I2C_PORTDELAY=$opt_d;}
 if ($opt_l ne "") { $I2C_LOG=$opt_l;}
 if ($opt_k ne "") { $I2C_LOG_SIZE=$opt_k;}
 if ($opt_b ne "") { $I2C_DEBUG=$opt_b;}
 return 1;
}

sub logmsg(@) { 
 print STDERR scalar(localtime())," I2C-Server: @_  \n"; 
}

sub check_log_size($ $) {
my $logfile = $_[0];
my $logsize = $_[1];

 if ((stat($logfile))[7] > $logsize) {
  system("mv $logfile $logfile.old ");
 }
 open(STDERR,">>$logfile") || die "Logfile kann nicht geoeffnet werden. $logfile \n";
}

